

AGlShaderText plain_colour_text = {" \
#version 130\n \
attribute vec2 position; \
uniform vec2 modelview; \
uniform vec2 translate; \
void main () \
{ \
 gl_Position = vec4(vec2(1., -1.) * (position + translate) / modelview - vec2(1.0, -1.0), 1.0, 1.0); \
} \
",
" \
uniform vec4 colour; \
void main (void) \
{ \
 gl_FragColor = colour; \
} \
"
};

AGlShaderText alpha_map_text = {" \
#version 130\n \
attribute vec4 vertex; \
uniform vec2 modelview; \
uniform vec2 translate; \
varying vec2 tex_coords; \
void main () \
{ \
   tex_coords = vertex.zw; \
   gl_Position = vec4(vec2(1., -1.) * (vertex.xy + translate) / modelview - vec2(1.0, -1.0), 0.0, 1.0); \
} \
",
" \
uniform sampler2D tex2d; \
uniform vec4 fg_colour; \
varying vec2 tex_coords; \
void main (void) \
{ \
 gl_FragColor = vec4(fg_colour.xyz, fg_colour.a * texture2D(tex2d, tex_coords).a); \
} \
"
};

AGlShaderText alpha_map_split_text = {" \
#version 130\n \
attribute vec2 vertex; \
attribute vec2 tvertex; \
uniform vec2 modelview; \
uniform vec2 translate; \
varying vec2 tex_coords; \
void main () \
{ \
   tex_coords = tvertex.xy; \
   gl_Position = vec4(vec2(1., -1.) * (vertex.xy + translate) / modelview - vec2(1.0, -1.0), 1.0, 1.0); \
} \
",
" \
uniform sampler2D tex2d; \
uniform vec4 fg_colour; \
varying vec2 tex_coords; \
void main (void) \
{ \
 gl_FragColor = vec4(fg_colour.xyz, fg_colour.a * texture2D(tex2d, tex_coords).a); \
} \
"
};

AGlShaderText texture_2d_text = {" \
#version 130\n \
attribute vec4 vertex; \
uniform vec2 modelview; \
uniform vec2 translate; \
varying vec2 tex_coords; \
void main () \
{ \
   tex_coords = vertex.zw; \
   gl_Position = vec4(vec2(1., -1.) * (vertex.xy + translate) / modelview - vec2(1.0, -1.0), 1.0, 1.0); \
} \
",
" \
uniform sampler2D tex2d; \
uniform vec4 fg_colour; \
varying vec2 tex_coords; \
void main (void) \
{ \
   gl_FragColor = fg_colour * texture2D(tex2d, tex_coords); \
} \
"
};

AGlShaderText dotted_text = {" \
#version 130\n \
uniform vec2 modelview; \
uniform vec2 translate; \
attribute vec2 position; \
varying vec2 MCposition; \
void main() \
{ \
   MCposition = position; \
   gl_Position = vec4(vec2(1., -1.) * (position + translate) / modelview - vec2(1.0, -1.0), 1.0, 1.0); \
} \
",
" \
uniform vec4 colour; \
varying vec2 MCposition; \
void main () \
{ \
 float interval = 2.0; \
 float m = mod(MCposition.x, interval); \
 float n = mod(MCposition.y, interval); \
 if(fract(MCposition.y / 3.0) > 0.6) \
  gl_FragColor = colour; \
 else \
  discard; \
} \
"
};

AGlShaderText rotatable_text = {" \
#version 130\n \
attribute vec4 vertex; \
uniform vec2 modelview; \
uniform vec2 translate; \
uniform float rotation; \
uniform vec2 origin; \
varying vec2 tex_coord; \
vec2 rotate (vec2 v, float a) \
{ \
   float s = sin(a); \
   float c = cos(a); \
   mat2 m = mat2(c, -s, s, c); \
   return m * v; \
} \
void main () \
{ \
 tex_coord = vertex.zw; \
 gl_Position = vec4( \
  vec2(1., -1.) * (rotate(vertex.xy - origin, rotation) + origin + translate) / modelview - vec2(1., -1.), \
  0., \
  1. \
 ); \
} \
",
" \
uniform sampler2D tex2d; \
uniform vec4 colour; \
varying vec2 tex_coord; \
void main (void) \
{ \
   gl_FragColor = colour * texture2D(tex2d, tex_coord); \
} \
"
};

AGlShaderText vscrollbar_text = {" \
#version 130\n \
uniform vec2 modelview; \
uniform vec2 translate; \
attribute vec2 vertex; \
varying vec2 position; \
void main () { \
 position = vertex; \
 gl_Position = vec4(vec2(1., -1.) * (vertex + translate) / modelview - vec2(1.0, -1.0), 1.0, 1.0); \
} \
",
" \
uniform vec2 centre1; \
uniform vec2 centre2; \
uniform float radius; \
uniform vec4 colour; \
uniform vec4 bg_colour; \
varying vec2 position; \
void main () { \
 float dist = 0.0; \
 if(position.y < centre1.y){ \
  dist = distance(position, centre1); \
 }else if(position.y > centre2.y){ \
  dist = distance(position, centre2); \
 }else{ \
  dist = distance(position.x, centre1.x); \
 } \
 gl_FragColor = mix(colour, bg_colour, smoothstep(radius - 1.0, radius + 1.0, dist)); \
} \
"
};

AGlShaderText hscrollbar_text = {" \
#version 130\n \
uniform vec2 modelview; \
uniform vec2 translate; \
attribute vec2 vertex; \
varying vec2 position; \
void main () { \
 position = vertex; \
 gl_Position = vec4(vec2(1., -1.) * (vertex + translate) / modelview - vec2(1.0, -1.0), 1.0, 1.0); \
} \
",
" \
uniform vec2 centre1; \
uniform vec2 centre2; \
uniform float radius; \
uniform vec4 colour; \
uniform vec4 bg_colour; \
varying vec2 position; \
void main () \
{ \
 float dist = 0.0; \
 if(position.x < centre1.x){ \
  dist = distance(position, centre1); \
 }else if(position.x > centre2.x){ \
  dist = distance(position, centre2); \
 }else{ \
  dist = distance(position.y, centre1.y); \
 } \
 gl_FragColor = mix(colour, bg_colour, smoothstep(radius - 1.0, radius + 1.0, dist)); \
} \
"
};
